/*
 *  Copyright (C) 2007 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

# ifdef USE_PSP_IRKEYB

#include <pspkernel.h>
#include <pspdebug.h>
#include <pspdisplay.h>
#include <pspctrl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>

#include "psp_irkeyb.h"
#include "danzeff.h"

//LUDO: HACK
struct _SDL_Joystick {
	Uint8 index;		/* Device index */
	const char *name;	/* Joystick name - system dependent */

	int naxes;		/* Number of axis controls on the joystick */
	Sint16 *axes;		/* Current axis states */

	int nhats;		/* Number of hats on the joystick */
	Uint8 *hats;		/* Current hat states */
	
	int nballs;		/* Number of trackballs on the joystick */
	struct balldelta {
		int dx;
		int dy;
	} *balls;		/* Current ball motion deltas */
	
	int nbuttons;		/* Number of buttons on the joystick */
	Uint8 *buttons;		/* Current button states */
	
	struct joystick_hwdata *hwdata;	/* Driver dependent information */

	int ref_count;		/* Reference count for multiple opens */
};

  static int loc_psp_irkeyb_mode = 0;

//#define CONFIG_FILE "ms0:/seplugins/pikey/pspirkeyb.ini"
#define KERNELMODE   0  /* 1 is untested but required for some keyboards to change baudrate */

/*
    PSP_IRKBD_OUTPUT_MODE_ASCII
    PSP_IRKBD_OUTPUT_MODE_RAW
    PSP_IRKBD_OUTPUT_MODE_SCANCODE
    PSP_IRKBD_OUTPUT_MODE_VT100    
  */

int
psp_irkeyb_init()
{
  loc_psp_irkeyb_mode = 0;

#ifdef PSPFW30X
  /* Load irda PRX for CFW >= 3.80 */
  u32 mod_id = sceKernelLoadModule("flash0:/kd/irda.prx", 0, NULL);
  sceKernelStartModule(mod_id, 0, NULL, NULL, NULL);
#endif

  int ret = pspIrKeybInit( "./pspirkeyb.ini", KERNELMODE );

  if( ret == PSP_IRKBD_RESULT_OK ) {
    pspIrKeybOutputMode( PSP_IRKBD_OUTPUT_MODE_VT100 );
    loc_psp_irkeyb_mode = 1;
    return 0;

  } else {
    switch( ret )
    {
      case PSP_IRKBD_RESULT_CANT_OPEN_DEVICE:
          fprintf(stdout,  "error: can't open device\n" );
          break;
      case PSP_IRKBD_RESULT_CANT_OPEN_MAPFILE:
          fprintf(stdout,  "error: can't open mapfile\n" );
          break;
      case PSP_IRKBD_RESULT_MAPFILE_MAXDEPTHLEVEL:
          fprintf(stdout,  "error: mapfile max include level reached - recursion?\n" );
          break;
      case PSP_IRKBD_RESULT_CANT_OPEN_MAPFILE_INCLUDE:
          fprintf(stdout,  "error: can't open include in mapfile\n" );
          break;
      case PSP_IRKBD_RESULT_CANT_SET_BAUDTATE:
          fprintf(stdout,  "error: can't set baudrate - you need kernel support\n" );
          break; 
      case PSP_IRKBD_RESULT_CONFIG_FILE_NOT_FOUND:
          fprintf(stdout,  "error: can't read config file\n" );
          break; 
      case PSP_IRKBD_RESULT_UNKNOW_KEYBOARD:
          fprintf(stdout,  "error: unknown keyboard\n" );
          break;
      case PSP_IRKBD_RESULT_FAILED:
      default:
          fprintf(stdout,  "error: init failed\n" );
          break;
    }
  }
  return 1;
}

 static u8  loc_irkbd_buffer[ 512 ];
 static int loc_irkbd_length  = 0;
 static int loc_irkbd_current = 0;
 static int loc_irkbd_inc_step = 0;

static int
psp_irkeyb_check_key()
{
  int key = PSP_IRKEYB_EMPTY;
  loc_irkbd_inc_step = 0;
  if (loc_psp_irkeyb_mode) {
    if (loc_irkbd_current >= loc_irkbd_length) {
      loc_irkbd_current = 0;
      loc_irkbd_length  = 0;
      pspIrKeybReadinput(loc_irkbd_buffer, &loc_irkbd_length);
    }
    int delta = loc_irkbd_length - loc_irkbd_current;
    if (delta >= 3) {
      u8* scan_buffer = &loc_irkbd_buffer[loc_irkbd_current];
      if (!strncmp(scan_buffer, "\033[B", 3)) {
        key = PSP_IRKEYB_DOWN;
        loc_irkbd_inc_step = 3;
      } else 
      if (!strncmp(scan_buffer, "\033[A", 3)) {
        key = PSP_IRKEYB_UP;
        loc_irkbd_inc_step = 3;
      } else 
      if (!strncmp(scan_buffer, "\033[C", 3)) {
        key = PSP_IRKEYB_RIGHT;
        loc_irkbd_inc_step = 3;
      } else
      if (!strncmp(scan_buffer, "\033[D", 3)) {
        key = PSP_IRKEYB_LEFT;
        loc_irkbd_inc_step = 3;
      }
    }
    if (delta >= 1) { 
      if (key == PSP_IRKEYB_EMPTY) {

        key = loc_irkbd_buffer[loc_irkbd_current];
        if ((key == 9) || (key == 23)) key = PSP_IRKEYB_START; //W
        else
        if ((key == 0x1b) || (key == 17)) key = PSP_IRKEYB_SELECT; //Q
        else
        if (key == 5) key = PSP_IRKEYB_TRIANGLE; //E
        else
        if (key == 19) key = PSP_IRKEYB_SQUARE; //S
        else
        if (key == 24) key = PSP_IRKEYB_CROSS; //X
        else
        if (key == 6) key = PSP_IRKEYB_CIRCLE; //F
        else
        if (key == 26) key = PSP_IRKEYB_LTRIGGER; //Z
        else
        if (key == 3) key = PSP_IRKEYB_RTRIGGER; //R

        loc_irkbd_inc_step = 1;
      }
    }
  }
# if 0 //LUDO: DEBUG
  if (key != PSP_IRKEYB_EMPTY) {
    fprintf(stdout, "press %x %c\n", key, key);
  }
# endif
  return key;
}

static void
psp_irkeyb_inc_buffer()
{
  if (loc_irkbd_inc_step) {
    loc_irkbd_current += loc_irkbd_inc_step;
    loc_irkbd_inc_step = 0;
  }
}

# if 0
int
psp_irkeyb_read_key()
{
  int psp_irkeyb = psp_irkeyb_check_key();
  if (psp_irkeyb < 0) psp_irkeyb = PSP_IRKEYB_EMPTY;
  else {
    psp_irkeyb_inc_buffer();
  }
  return psp_irkeyb;
}
# endif

static int
loc_psp_irkeyb_read_key()
{
  int psp_irkeyb = psp_irkeyb_check_key();
  if (psp_irkeyb < 0) psp_irkeyb = PSP_IRKEYB_EMPTY;
  else {
    psp_irkeyb_inc_buffer();
  }
  return psp_irkeyb;
}

int
psp_irkeyb_read_key()
{
  int psp_irkeyb = loc_psp_irkeyb_read_key();
  if (psp_irkeyb != PSP_IRKEYB_EMPTY) {
    if ((psp_irkeyb == 0xc2) || (psp_irkeyb == 0xc3)) {
      u8 c1 = psp_irkeyb;
      int new_key = loc_psp_irkeyb_read_key();
      if (new_key != PSP_IRKEYB_EMPTY) {
        u8 c2 = new_key;
        psp_irkeyb = psp_convert_utf8_to_iso_8859_1(c1, c2);
      }
    }
  }
  return psp_irkeyb;
}


/* HACK 

    PSP_CTRL_TRIANGLE       0
    PSP_CTRL_CIRCLE         1
    PSP_CTRL_CROSS          2
    PSP_CTRL_SQUARE         3
    PSP_CTRL_LTRIGGER       4
    PSP_CTRL_RTRIGGER       5
    PSP_CTRL_DOWN           6
    PSP_CTRL_LEFT           7
    PSP_CTRL_UP             8
    PSP_CTRL_RIGHT          9
    PSP_CTRL_SELECT        10
    PSP_CTRL_START         11
    PSP_CTRL_HOME          12
    PSP_CTRL_HOLD          13

*/

static int loc_pending_buttons[12];

int
psp_irkeyb_set_psp_key(SDL_Joystick* joy)
{
  int index;
  int joy_id;

  int prev_inc_step = loc_irkbd_inc_step;

  for (index = 0; index < 12; index++) {
    if (loc_pending_buttons[index]) {
      joy->buttons[index]= SDL_RELEASED;
      loc_pending_buttons[index] = 0;
    }
  }

  int irkeyb_key = psp_irkeyb_check_key();

  if (irkeyb_key != PSP_IRKEYB_EMPTY) {

    if (irkeyb_key == PSP_IRKEYB_TRIANGLE) joy_id = 0;
    else
    if (irkeyb_key == PSP_IRKEYB_CIRCLE) joy_id = 1;
    else
    if (irkeyb_key == PSP_IRKEYB_CROSS) joy_id = 2;
    else
    if (irkeyb_key == PSP_IRKEYB_SQUARE) joy_id = 3;
    else
    if (irkeyb_key == PSP_IRKEYB_LTRIGGER) joy_id = 4;
    else
    if (irkeyb_key == PSP_IRKEYB_RTRIGGER) joy_id = 5;
    else
    if (irkeyb_key == PSP_IRKEYB_DOWN) joy_id = 6;
    else 
    if (irkeyb_key == PSP_IRKEYB_LEFT) joy_id = 7;
    else
    if (irkeyb_key == PSP_IRKEYB_UP) joy_id = 8;
    else 
    if (irkeyb_key == PSP_IRKEYB_RIGHT) joy_id = 9;
    else 
    if (irkeyb_key == PSP_IRKEYB_SELECT) joy_id = 10;
    else
    if (irkeyb_key == PSP_IRKEYB_START) joy_id = 11;
    else {
       if (prev_inc_step) {
         /* drop it ! */
         loc_irkbd_inc_step = prev_inc_step;
         psp_irkeyb_inc_buffer();
       }
       return PSP_IRKEYB_EMPTY;
    }
    joy->buttons[joy_id] = SDL_PRESSED;
    loc_pending_buttons[joy_id] = 1;
    psp_irkeyb_inc_buffer();
  }
  return irkeyb_key;
}

void
psp_irkeyb_exit()
{
  if (loc_psp_irkeyb_mode) {
    loc_psp_irkeyb_mode = 0;
    pspIrKeybFinish();
  }
}

# endif
